@@ -794,6 +794,7 @@ def decay_rate(sim, field_index=None, wavelength=None,
         ## electric LDOS
         Q = sim.dyads.G_EE
         S = sim.dyads.G_EE
+        Q_sign = +1
         calc_H = 0
         ## exclude self-term (assume homogeneous mesh and environment)
         selfterm = sim.dyads.getSelfTermEE(wavelength, sim.struct)[0]
@@ -801,6 +802,7 @@ def decay_rate(sim, field_index=None, wavelength=None,
         ## magnetic LDOS
         Q = sim.dyads.G_HE
         S = sim.dyads.G_HE
+        Q_sign = -1
         calc_H = 0
         ## exclude self-term (assume homogeneous mesh and environment)
         selfterm = sim.dyads.getSelfTermHE(wavelength, sim.struct)[0]
@@ -888,7 +890,7 @@ def decay_rate(sim, field_index=None, wavelength=None,
         
         ## perform double volume integral for each probe position --> numba accelerated
         t5 = time.time()
-        _do_double_integral(KE2, Q_list, S_list, alpha_tensor, _Sp_out)
+        _do_double_integral(KE2, Q_sign*Q_list, S_list, alpha_tensor, _Sp_out)
         Sp[idx_r_outside] = _Sp_out
         if verbose: 
             print("integrate: {:.1f}s, ".format((time.time() - t5)), end='')
