@@ -1,6 +1,6 @@
 # encoding: utf-8
 #
-#Copyright (C) 2017-2021, P. R. Wiecha
+#Copyright (C) 2017-2022, P. R. Wiecha
 #
 #This program is free software: you can redistribute it and/or modify
 #it under the terms of the GNU General Public License as published by
@@ -145,12 +145,13 @@ def extinct(sim, field_index, with_radiation_correction=False, normalization_E0=
     Journal of the Optical Society of America, A 11, 1491 (1994).
     
     """
+    from pyGDM2 import tools 
     if sim.E is None: 
         raise ValueError("Error: Scattered field inside the structure not yet evaluated. Run `core.scatter` simulation first.")
     
     ## --- incident field configuration
-    field_params    = sim.E[field_index][0]
-    wavelength      = field_params['wavelength']
+    field_params = tools.get_field_indices(sim)[field_index]
+    wavelength   = field_params['wavelength']
     sim.struct.setWavelength(wavelength)
     eps_env = sim.dyads.getEnvironmentIndices(wavelength, sim.struct.geometry[:1])[0]  # assume structure is fully in one environment
     n_env = np.sqrt(eps_env)
@@ -289,10 +290,12 @@ def farfield(sim, field_index,
 # =============================================================================
 #     preparation
 # =============================================================================
+    from pyGDM2 import tools
+
     ## --- spherical probe coordinates
     if r_probe is None:
-        tetalist = np.ones((int(Nteta), int(Nphi)))*np.linspace(tetamin, tetamax, Nteta)[:,None]
-        philist = np.ones((int(Nteta), int(Nphi)))*np.linspace(phimin, phimax, Nphi, endpoint=False)[None,:]
+        tetalist = np.ones((int(Nteta), int(Nphi)))*np.linspace(tetamin, tetamax, int(Nteta))[:,None]
+        philist = np.ones((int(Nteta), int(Nphi)))*np.linspace(phimin, phimax, int(Nphi), endpoint=False)[None,:]
         xff = (r * np.sin(tetalist) * np.cos(philist)).flatten()
         yff = (r * np.sin(tetalist) * np.sin(philist)).flatten()
         zff = (r * np.cos(tetalist)).flatten()
@@ -301,11 +304,11 @@ def farfield(sim, field_index,
         _r_probe = r_probe
         
     ## --- spherical integration steps
-    dteta = (tetamax-tetamin)/float(Nteta-1)
-    dphi = 2.*np.pi/float(Nphi)
+    dteta = (tetamax-tetamin) / float(Nteta-1)
+    dphi = (phimax-phimin) / float(Nphi)
     
     ## --- incident field config
-    field_params    = sim.E[field_index][0]
+    field_params    = tools.get_field_indices(sim)[field_index]
     wavelength      = field_params['wavelength']
     
     ## --- environment
@@ -401,7 +404,7 @@ def farfield(sim, field_index,
         if r_probe is None:
             return tetalist, philist, Escat, Escat + E0, E0
         else:
-            return I_sc, I_tot, I0
+            return Escat, Escat + E0, E0
     else:
         d_solid_surf = r**2 * np.sin(tetalist) * dteta * dphi
         if return_value.lower() == 'int_es':
@@ -432,7 +435,8 @@ def farfield(sim, field_index,
 # nearfield functions
 # =============================================================================
 def nearfield(sim, field_index, r_probe, which_fields=["Es","Et","Bs","Bt"],
-                 val_inside_struct="field", N_neighbors_internal_field=4,
+                 val_inside_struct="field", val_outside_struct="field", 
+                 N_neighbors_internal_field=4,
                  method=''):
     """Nearfield distribution in proximity of nanostructre
     
@@ -468,6 +472,9 @@ def nearfield(sim, field_index, r_probe, which_fields=["Es","Et","Bs","Bt"],
         the field at the location of the closest meshpoint. Can be very coarse.
         Disable by setting "None", but note that inside the structure the 
         Green's function will diverge in the latter case.
+
+    val_outside_struct : str, default: "field"
+        see `val_inside_struct`
     
     N_neighbors_internal_field : int, default: 4
         Average internal field over field at *N* closes meshpoints. Neighbor 
@@ -569,9 +576,10 @@ def nearfield(sim, field_index, r_probe, which_fields=["Es","Et","Bs","Bt"],
 # =============================================================================
 #     treat positions inside structure
 # =============================================================================
-        val_inside_struct = str(val_inside_struct)
+        val_inside_struct = str(val_inside_struct).lower()
         
         _i_r_probe_out = []
+        _i_r_probe_in = []
         _r_probe_out = []
         if val_inside_struct.lower() != "none":
             from scipy.linalg import norm
@@ -604,6 +612,8 @@ def nearfield(sim, field_index, r_probe, which_fields=["Es","Et","Bs","Bt"],
                         Escat[i] = fill_valueE
                     if "bs" in which_fields or "bt" in which_fields or "hs" in which_fields or "ht" in which_fields:
                         Bscat[i] = fill_valueB
+                        
+                    _i_r_probe_in.append(i)
                 ## --- not inside, requires field propagation
                 else:
                     _i_r_probe_out.append(i)
@@ -611,50 +621,72 @@ def nearfield(sim, field_index, r_probe, which_fields=["Es","Et","Bs","Bt"],
         
         _r_probe_out = np.array(_r_probe_out)
         _i_r_probe_out = np.array(_i_r_probe_out)
+        _i_r_probe_in = np.array(_i_r_probe_in)
             
 # =============================================================================
 #     evaluate Green's function and propagate fields
 # =============================================================================
+        val_outside_struct = str(val_outside_struct).lower()
+        
         if len(_r_probe_out) > 0:
             ## +++++++++++++ electric field +++++++++++++
             if "es" in which_fields or "et" in which_fields:
-                ## Greens function for each dipole
-                G_NF = np.zeros((len(sim.struct.geometry), len(_r_probe_out), 3, 3), 
-                                dtype = sim.efield.dtypec)
-                sim.dyads.eval_G(sim.struct.geometry, _r_probe_out, 
-                                 sim.dyads.G_EE, wavelength, conf_dict, G_NF)
-                
-                ## propagate fields 
-                G_NF = np.moveaxis(G_NF, 0,2).reshape((len(_r_probe_out), 3, -1))  # re-arrange for direct matrix-vector multiplication
-                _Escat_out = np.zeros(shape=(len(_r_probe_out), 3), dtype=sim.efield.dtypec)
-                _calc_repropagation(P, _Escat_out, G_NF)
-                
-                ##  fill final field-list with outside values
-                Escat[_i_r_probe_out] = _Escat_out
-                
+                if val_outside_struct == 'field':
+                    ## Greens function for each dipole
+                    G_NF = np.zeros((len(sim.struct.geometry), len(_r_probe_out), 3, 3), 
+                                    dtype = sim.efield.dtypec)
+                    sim.dyads.eval_G(sim.struct.geometry, _r_probe_out, 
+                                     sim.dyads.G_EE, wavelength, conf_dict, G_NF)
+                    
+                    ## propagate fields 
+                    G_NF = np.moveaxis(G_NF, 0,2).reshape((len(_r_probe_out), 3, -1))  # re-arrange for direct matrix-vector multiplication
+                    _Escat_out = np.zeros(shape=(len(_r_probe_out), 3), dtype=sim.efield.dtypec)
+                    _calc_repropagation(P, _Escat_out, G_NF)
+                    
+                    ##  fill final field-list with outside values
+                    Escat[_i_r_probe_out] = _Escat_out
+                elif val_outside_struct == "nan":
+                    Escat[_i_r_probe_out] = np.nan
+                else:
+                    Escat[_i_r_probe_out] = 0
+                    
             ## +++++++++++++ magnetic field +++++++++++++
-            if "bs" in which_fields or "bt" in which_fields or "hs" in which_fields or "ht" in which_fields:
-                ## Greens function for each dipole
-                G_NF = np.zeros((len(sim.struct.geometry), len(_r_probe_out), 3, 3), 
-                                dtype = sim.efield.dtypec)
-                sim.dyads.eval_G(sim.struct.geometry, _r_probe_out, 
-                                 sim.dyads.G_HE, wavelength, conf_dict, G_NF)
-                
-                ## propagate fields 
-                G_NF = np.moveaxis(G_NF, 0,2).reshape((len(_r_probe_out), 3, -1))  # re-arrange for direct matrix-vector multiplication
-                _Bscat_out = np.zeros(shape=(len(_r_probe_out), 3), dtype=sim.efield.dtypec)
-                _calc_repropagation(P, _Bscat_out, G_NF)
-                
-                ##  fill final field-list with outside values
-                Bscat[_i_r_probe_out] = _Bscat_out
+            if ("bs" in which_fields or "bt" in which_fields or "hs" in which_fields 
+                    or "ht" in which_fields):
+                if val_outside_struct == 'field':
+                    ## Greens function for each dipole
+                    G_NF = np.zeros((len(sim.struct.geometry), len(_r_probe_out), 3, 3), 
+                                    dtype = sim.efield.dtypec)
+                    sim.dyads.eval_G(sim.struct.geometry, _r_probe_out, 
+                                     sim.dyads.G_HE, wavelength, conf_dict, G_NF)
+                    
+                    ## propagate fields 
+                    G_NF = np.moveaxis(G_NF, 0,2).reshape((len(_r_probe_out), 3, -1))  # re-arrange for direct matrix-vector multiplication
+                    _Bscat_out = np.zeros(shape=(len(_r_probe_out), 3), dtype=sim.efield.dtypec)
+                    _calc_repropagation(P, _Bscat_out, G_NF)
+                    
+                    ##  fill final field-list with outside values
+                    Bscat[_i_r_probe_out] = _Bscat_out
+                elif val_outside_struct == "nan":
+                    Bscat[_i_r_probe_out] = np.nan
+                else:
+                    Bscat[_i_r_probe_out] = 0
     
 # =============================================================================
 #     bundle output
 # =============================================================================
         if "et" in which_fields:
-            Etot = Escat + E0
+            Etot = Escat.copy()
+            if val_outside_struct == 'field' and len(_i_r_probe_out)>0:
+                Etot[_i_r_probe_out] = Etot[_i_r_probe_out] + E0[_i_r_probe_out]
+            if val_inside_struct == 'field' and len(_i_r_probe_in)>0:
+                Etot[_i_r_probe_in] = Etot[_i_r_probe_in] + E0[_i_r_probe_in]
         if "bt" in which_fields or "ht" in which_fields:
-            Btot = Bscat + B0
+            Btot = Bscat.copy()
+            if val_outside_struct == 'field' and len(_i_r_probe_out)>0:
+                Btot[_i_r_probe_out] = Btot[_i_r_probe_out] + B0[_i_r_probe_out]
+            if val_inside_struct == 'field' and len(_i_r_probe_in)>0:
+                Btot[_i_r_probe_in] = Btot[_i_r_probe_in] + B0[_i_r_probe_in]
     
     
     return_field_list = []
@@ -786,8 +818,7 @@ def poynting(sim, field_index, r_probe, which_fields="tot"):
 # field gradients
 # =============================================================================
 def field_gradient(sim, field_index, 
-                   r_probe=None, delta=None, 
-                   which_fields=["Et"]):
+                   r_probe=None, delta=None, which_fields=["Et"]):
     """nearfield gradient distribution inside or in proximity of nanostructure
     
     Calculate field-gradients (positions defined by `r_probe`). 
@@ -860,17 +891,19 @@ def field_gradient(sim, field_index,
                          "elements, either floats, or lists.")
     r_probe = np.transpose(r_probe)
     
-    if sim.E is None: 
-        raise ValueError("Error: Scattered field inside the structure not yet evaluated. Run `core.scatter` simulation first.")
+    # if sim.E is None: 
+    #     raise ValueError("Error: Scattered field inside the structure not yet evaluated. Run `core.scatter` simulation first.")
     
     
     
 # =============================================================================
 #     preparation
 # =============================================================================
+    from pyGDM2 import tools
+    
     which_fields = [wf.lower() for wf in which_fields]
     
-    field_kwargs = sim.E[field_index][0]
+    field_kwargs = tools.get_field_indices(sim)[field_index]
     wavelength = field_kwargs['wavelength']
     sim.struct.setWavelength(wavelength)    
     
