@@ -1444,11 +1444,13 @@ def _test_availability_generalized_polarizability(sim, which_moment, wavelength,
     
 
 
-def density_of_multipolar_modes(sim, field_index=None, wavelength=None, 
-                                which_moments=['p1', 'pt', 'qe1', 'qt', 'm', 'qm'], 
+def density_of_multipolar_modes(sim, field_index=None, wavelength=None, return_mode_energy=True, 
+                                which_moments=['p', 'm', 'qe', 'qm'], 
                                 method='lu', long_wavelength_approx=False):
     """Calculate total density of multipole modes via generalized polarizability
     
+    If not yet calculated, will invoke generalize pola. calculation, thus it may
+    require some computation time on the first call.
     
 
     Parameters
@@ -1460,6 +1462,10 @@ def density_of_multipolar_modes(sim, field_index=None, wavelength=None,
         index of evaluated self-consistent field to use for calculation. Can be
         obtained for specific parameter-set using :func:`.tools.get_closest_field_index`
     
+    energy_density : bool, optional
+        whether to return the total mode energy (True) or the accumulated 
+        multipole amplitude norm (False). The default is True.
+    
     wavelength : float, default: None
         Optional wavelength (alternative to `field_index`) at which to 
         calculate susceptibility matrix (in nm). 
@@ -1490,6 +1496,7 @@ def density_of_multipolar_modes(sim, field_index=None, wavelength=None,
         total density of modes for each demanded order (= sum of tensor norms 
         of all generalized polarizabilities for each multipole)
     
+    
     Notes
     -----
     For details , see: 
@@ -1519,28 +1526,28 @@ def density_of_multipolar_modes(sim, field_index=None, wavelength=None,
         _test_availability_generalized_polarizability(sim, which_moment, 
                                                       wavelength, method=method, 
                                                       long_wavelength_approx=long_wavelength_approx)
-    
+    exp_order = 2 if return_mode_energy else 1
     
     out_list = []
     for wm in which_moments:
         if 'p' == wm:
-            out_list.append(np.sum(np.linalg.norm(sim.K_P_E[wavelength], axis=(1,2))) + 
-                            np.sum(np.linalg.norm(sim.K_T_E[wavelength], axis=(1,2))))
+            out_list.append((np.sum(np.linalg.norm(sim.K_P_E[wavelength], axis=(1,2))) + 
+                            np.sum(np.linalg.norm(sim.K_T_E[wavelength], axis=(1,2))))**exp_order)
         if 'p1' == wm:
-            out_list.append(np.sum(np.linalg.norm(sim.K_P_E[wavelength], axis=(1,2))))
+            out_list.append(np.sum(np.linalg.norm(sim.K_P_E[wavelength], axis=(1,2)))**exp_order)
         if 'pt' == wm:
-            out_list.append(np.sum(np.linalg.norm(sim.K_T_E[wavelength], axis=(1,2))))
+            out_list.append(np.sum(np.linalg.norm(sim.K_T_E[wavelength], axis=(1,2)))**exp_order)
         if 'qe' == wm:
-            out_list.append(np.sum(np.linalg.norm(sim.K_QE_E[wavelength], axis=(1,2))) +
-                            np.sum(np.linalg.norm(sim.K_QT_E[wavelength], axis=(1,2))))
+            out_list.append((np.sum(np.linalg.norm(sim.K_QE_E[wavelength], axis=(1,2))) +
+                            np.sum(np.linalg.norm(sim.K_QT_E[wavelength], axis=(1,2))))**exp_order)
         if 'qe1' == wm:
-            out_list.append(np.sum(np.linalg.norm(sim.K_QE_E[wavelength], axis=(1,2))))
+            out_list.append(np.sum(np.linalg.norm(sim.K_QE_E[wavelength], axis=(1,2)))**exp_order)
         if 'qt' == wm:
-            out_list.append(np.sum(np.linalg.norm(sim.K_QT_E[wavelength], axis=(1,2))))
+            out_list.append(np.sum(np.linalg.norm(sim.K_QT_E[wavelength], axis=(1,2)))**exp_order)
         if 'm' == wm:
-            out_list.append(np.sum(np.linalg.norm(sim.K_M_E[wavelength], axis=(1,2))))
+            out_list.append(np.sum(np.linalg.norm(sim.K_M_E[wavelength], axis=(1,2)))**exp_order)
         if 'qm' == wm:
-            out_list.append(np.sum(np.linalg.norm(sim.K_QM_E[wavelength], axis=(1,2))))
+            out_list.append(np.sum(np.linalg.norm(sim.K_QM_E[wavelength], axis=(1,2)))**exp_order)
     
     return out_list
 
