@@ -212,7 +212,7 @@ def load_simulation(fname, pickle_encoding=None):
             try:
                 sim = pickle.load( open( fname, "rb" ) )
             except UnicodeDecodeError:
-                warnings.warn('Unicode error encountered during loading. Can be cause by python2-savefiles. Trying "latin1" encoding...')
+                warnings.warn('Unicode error encountered during loading. Can be caused by python2-savefiles. Trying "latin1" encoding...')
                 sim = pickle.load( open( fname, "rb" ), encoding='latin1' )
         else:
             sim = pickle.load( open( fname, "rb" ), encoding=pickle_encoding )
@@ -220,6 +220,7 @@ def load_simulation(fname, pickle_encoding=None):
         ## --- read E fields from hdf5
         f = tables.open_file(fname + '.h5', 'r')
         if "/E" in f:
+            if sim.E is None: sim.E = [[_d, None] for _d in get_field_indices(sim)]
             for ef in f.root.E:
                 enr = int(ef.name[1:])
                 E = ef.read()
@@ -227,6 +228,7 @@ def load_simulation(fname, pickle_encoding=None):
         
         ## --- read H fields from hdf5
         if "/H" in f:
+            if sim.H is None: sim.H = [[_d, None] for _d in get_field_indices(sim)]
             for hf in f.root.H:
                 hnr = int(hf.name[1:])
                 H = hf.read()
@@ -249,7 +251,8 @@ def load_simulation(fname, pickle_encoding=None):
         if os.path.isfile(fname + '.h5'):
             warnings.warn("hdf5 file seems to exists, but pytables is not installed. " +
                           "The simulation results in the file will probably not be reloaded.")
-    
+    except Exception as e:
+        raise e
     
     ## --- compatibility with old API
     if not hasattr(sim, 'dyads'):
