@@ -1,6 +1,6 @@
 # encoding: utf-8
 #
-#Copyright (C) 2017-2021, P. R. Wiecha
+#Copyright (C) 2017-2022, P. R. Wiecha
 #
 #This program is free software: you can redistribute it and/or modify
 #it under the terms of the GNU General Public License as published by
@@ -527,6 +527,10 @@ def vectorfield(NF, struct=None, projection='auto', complex_part='real',
         sort vectors by length to avoid clipping (True: Plot longest 
         vectors on top)
      
+    overrideMaxL : bool, default: False
+        if True, use 'scale' as absolute scaling. Otherwise, normalize to 
+        field-vector amplitude.
+     
     Returns
     -------
     
@@ -607,7 +611,11 @@ def vectorfield(NF, struct=None, projection='auto', complex_part='real',
     if sortL:
         X,Y,UX,UY = sortbylength(X,Y,UX,UY)
     
-    LenALL = np.sqrt(UX**2 + UY**2)
+    
+    ## vector length
+    LenALL = np.sqrt(UX**2 + UY**2)   # absolute scaling
+    
+    
     if not overrideMaxL:
         scale = scale * LenALL.max()
     else:
