@@ -1,6 +1,6 @@
 # encoding: utf-8
 #
-#Copyright (C) 2017-2021, P. R. Wiecha
+#Copyright (C) 2017-2022, P. R. Wiecha
 #
 #This program is free software: you can redistribute it and/or modify
 #it under the terms of the GNU General Public License as published by
@@ -239,7 +239,8 @@ def vectorfield(NF, struct=None, scale=1.5, abs_scale=False,
         NF = NF[1]
     
     if len(NF.T) == 6:
-        X,Y,Z, UXcplx,UYcplx,UZcplx = np.transpose(NF).real
+        X,Y,Z, UXcplx,UYcplx,UZcplx = np.transpose(NF)
+        X,Y,Z = X.real,Y.real,Z.real
     elif len(NF.T) == 3 and struct is not None:
         UXcplx,UYcplx,UZcplx = np.transpose(NF)
         X,Y,Z = tools.get_geometry(struct).real
